//------------------------------------------------------------------------------
// NumberEncoding.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: keithkin
//
// <header>	$NumberEncoding$</header>
//------------------------------------------------------------------------------

//Exceptions that can be thrown
var L_integerencodingoutofrange_text = "VEIntegerEncoding: The number encoded is out of supported range";
var L_floatintegermapencodingoutofrange_text = "VEFloatIntegerMap: The number encoded is out of supported range";
var L_integerencodinginvalidstringlength_text = "VEIntegerEncoding: Invalid string length";
var L_integerencodingunknowndigit_text = "VEIntegerEncoding: The encoded string has an unknown digit";

//VEIntegerEncoding
//   Encodes an integer as a string using base N notation with the passed in digits to use as characters.
//   The algorithm is from BBeatty.
//
//   interface:
//      VEIntegerEncoding(digits, valueLength)
//         constructor
//         digits: string of characters to use in the string encoding. Each char must be unique.
//         valueLength: the number of encoding chars to use in the string when encoding
//   
//      MaxValue()
//         returns max possible integer value
// 
//      ValueLength()
//         returns length of encoded value
//
//      Encode(numValue)
//         numValue: integer
//         encodes numvalue and returns encoded string
//         throws exception if numValue is out of range
//
//      Decode(str)
//         str: encoded string
//         decodes string value and returns decoded integer
//         throws exception if there is an invalid digit in the string
//         throws exception if string is incorrect length
//
//		DigitValue(digit)
//         digit: encoded digit
//         returns decoded integer value for digit.
//         throws exception if digit is not in the hashtable
//
function VEIntegerEncoding(digits, valueLength)
{
	var m_digits = digits;
	var m_base = digits.length;
	var m_valueLength = valueLength;
	
	//calculate the max value that can be stored
	var power=1;
	for (var i=0; i < m_valueLength; ++i)
	{
		power *= m_base;
	}
	var m_maxValue = power - 1;
	
	//map from digits to values
	var m_digitMap = new Array();
	for (var digitValue=0; digitValue < m_digits.length; ++digitValue)
	{
		m_digitMap[m_digits.substr(digitValue,1)] = digitValue;
	}

	//MaxValue
	this.MaxValue = function()
	{
		return m_maxValue;
	}
	
	//ValueLength
	this.ValueLength = function()
	{
		return m_valueLength;
	}
	
	//Encode
	this.Encode = function(numValue)
	{
		if (numValue <= m_maxValue)
		{
			var encodedString = "";

			//create an array of digits
			var digitValues = new Array();
			
			//initialize the digit values in the array to 0
			for (var i=0; i < m_valueLength; ++i)
			{
				digitValues[i] = 0;
			}

			//populate digits from the provided value. Start from least-significant to most-significant
			var digitIndex = m_valueLength - 1;
			while (numValue > 0)
			{
				digitValues[digitIndex] = Math.floor(numValue % m_base);
				numValue = Math.floor(numValue / m_base);
				--digitIndex;
			}

			//Build the encoded string from the provided value
			for (var i=0; i < digitValues.length; ++i)
			{
				encodedString += m_digits.substr(digitValues[i],1);
			}
			
			return encodedString;
		}
		else
		{
			//out of range exception
			throw L_integerencodingoutofrange_text;
		}
	}
	
	//Decode
	this.Decode = function(strValue)
	{
		if (strValue.length == m_valueLength)
		{
			var numValue = 0;
			for( var i=0; i < strValue.length; ++i)
			{
				numValue *= m_base;
				numValue += this.DigitValue(strValue.substr(i,1));
			}
			
			return numValue;
		}
		else
		{
			//invalid string length exception
			throw L_integerencodinginvalidstringlength_text;
		}
	}
	
	//DigitValue
	this.DigitValue = function(digit)
	{
		if ((m_digitMap[digit] != null) && (m_digitMap[digit] != "undefined"))
		{
			return m_digitMap[digit];
		}
		else
		{
			//Unknown digit exception
			throw L_integerencodingunknowndigit_text;
		}
	}
}

//VEFloatIntegerMap
//   Maps a range of floating-point values to range of integers
//   The algorithm is from BBeatty.
//
//   interface:
//      VEFloatIntegerMap(minFloat, maxFloat, maxInt)
//         constructor
//            minFloat: min float value
//            maxFloat: max float value
//            maxInt: max int value
//   
//      MinFloat()
//         returns min possible float value
// 
//      MaxFloat()
//         returns max possible float value
//
//      MaxInt()
//         returns max possible int value
//
//      FloatToInt(floatValue)
//         floatValue: floating point number
//         returns mapped integer value from floating point value 
//         throws exception if floatValue is out of range
//
//      IntToFloat(intValue)
//         intValue: integer
//         returns mapped floating point value from passed in int value
//         throws exception if intValue is out of range
//
function VEFloatIntegerMap(minFloat, maxFloat, maxInt)
{
	var m_minFloat = minFloat;
	var m_maxFloat = maxFloat;
	var m_maxInt = maxInt;
	
	this.MinFloat = function() { return m_minFloat; }
	this.MaxFloat = function() { return m_maxFloat; }
	this.MaxInt = function() { return m_maxInt; }
	
	//FloatToInt
	this.FloatToInt = function(floatValue)
	{
		if ((floatValue >= m_minFloat) && (floatValue <= m_maxFloat))
		{
			var fraction = (floatValue - m_minFloat) / (m_maxFloat - m_minFloat);
			var intValue = (fraction * m_maxInt) + 0.5;
			return Math.min(Math.floor(intValue), m_maxInt);
		}
		else
		{
			//Out of range exception
			throw L_floatintegermapencodingoutofrange_text;
		}
	}
	
	//IntToFloat
	this.IntToFloat = function(intValue)
	{
		if (intValue <= m_maxInt)
		{
			var fraction = intValue / m_maxInt;
			var floatNumber = m_minFloat + fraction * (m_maxFloat - m_minFloat);
			return floatNumber;
		}
		else
		{
			//Out of range exception
			throw L_floatintegermapencodingoutofrange_text;
		}	
	}
}
