//------------------------------------------------------------------------------
// MapControl/MapView.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// A view consists of a lat/long, a zoom level and a map style
Msn.VE.MapView = function()
{
	this.zoomLevel = 0;
	this.mapStyle = null;
	this.center = new Msn.VE.Pixel();
	this.latlong = new Msn.VE.LatLong();
	this.pixelRect = new Msn.VE.PixelRectangle();
	this.latlongRect = new Msn.VE.LatLongRectangle();
	this.sceneId = null;
	this.sceneOrientation = null;
	
	// Private accessor to "this";
	var p_this = this;
	
	// The view type tracks what the defining type of the view is.  It can have one of
	// the following values: pixel, latlong,  pixelrect, latlongrect.
	// Pixel is the default and the preferred view type.  When setting a view, the map control
	// will always first try to resolve the view into a pixel view type.
	var viewType = "pixel";
	
	// Destroys this view, frees up object references.
	this.Destroy = function()
	{
		this.center = this.latlong = p_this = null;
	};
	
	this.GetViewType = function()
	{
		return viewType;
	};
	
	// Makes a new View object with the same contents.
	function MakeCopy()
	{
		var copy = new Msn.VE.MapView();
		copy.Copy(p_this);
		return copy;
	}
	
	// Copies the values of another view to this view.
	function Copy(view)
	{
		p_this.zoomLevel = view.zoomLevel;
		p_this.mapStyle = view.mapStyle;
		p_this.center.Copy(view.center);
		p_this.latlong.Copy(view.latlong);
		p_this.pixelRect.Copy(view.pixelRect);
		p_this.latlongRect.Copy(view.latlongRect);
		p_this.sceneId = view.sceneId;
		p_this.sceneOrientation = view.sceneOrientation;
		viewType = view.GetViewType();
	}
	
	// Tests for equality with another view object.  This is only applicable
	// when viewType = pixel.
	function Equals(view)
	{
		return view != null &&
			p_this.zoomLevel == view.zoomLevel &&
			p_this.mapStyle == view.mapStyle && 
			MathAbs(p_this.center.x - view.center.x) < 0.000001 &&
			MathAbs(p_this.center.y - view.center.y) < 0.000001 &&
			p_this.sceneId == view.sceneId &&
			p_this.sceneOrientation == view.sceneOrientation;
	}
	
	// Creates a string representation of this view object.
	function ToString()
	{
		return "(" + p_this.latlong.ToString() + ", " + p_this.zoomLevel + ", " + p_this.mapStyle + ")";
	}
	
	// Sets the center pixel.  Also updates the center latlong using the currentMode.
	function SetCenter(pixel)
	{
		if (!pixel)
		{
			return;
		}
		
		p_this.center = pixel;
		viewType = "pixel";
	}
	
	// Sets the center latlong.  Also updates the center pixel using the currentMode.
	function SetCenterLatLong(latlong)
	{
		if (!latlong)
		{
			return;
		}
		
		p_this.latlong = latlong;
		viewType = "latlong";
	}
	
	function SetPixelRectangle(pixelRect)
	{
		p_this.pixelRect = pixelRect;
		viewType = "pixelRect";
	}
	
	function SetLatLongRectangle(latlongRect)
	{
		p_this.latlongRect = latlongRect;
		viewType = "latlongRect";
	}
	
	// Sets the zoom level.  Updates the center pixel by scaling accordingly.
	// The center latlong should not need to change.
	function SetZoomLevel(zoomLevel)
	{
		// Zoom levels can never be less than or equal to zero.
		// If the zoom level drops below 1, set it to the min zoom level which is 1.
		if (zoomLevel <= 0)
		{
			zoomLevel = 1;
		}
		
		switch (viewType)
		{
			case "pixel":
				var factor = Math.pow(2, zoomLevel - p_this.zoomLevel);
				p_this.center.x *= factor;
				p_this.center.y *= factor;
				break;
				
			case "pixelRect":
				var factor = Math.pow(2, zoomLevel - p_this.zoomLevel);
				p_this.pixelRect.topLeft.x *= factor;
				p_this.pixelRect.topLeft.y *= factor;
				p_this.pixelRect.bottomRight.x *= factor;
				p_this.pixelRect.bottomRight.y *= factor;
				break;
		}
		
		p_this.zoomLevel = zoomLevel;
	}
	
	function SetMapStyle(mapStyle, sceneId, sceneOrientation)
	{
		p_this.mapStyle = mapStyle;
		
		if (viewType == "pixel")
		{
			viewType = "latlong";
		}
		
		if (mapStyle != 'o')
		{
			p_this.sceneId = null;
			p_this.sceneOrientation = null
		}
		else
		{
			p_this.sceneId = sceneId;
			p_this.sceneOrientation = sceneOrientation;
		}
	}
	
	// Gets the x at a zoom level.  This involves scaling the value by a power of 2.
	function GetX(zoomLevel)
	{
		if (zoomLevel)
		{
			return p_this.center.x * Math.pow(2, zoomLevel - p_this.zoomLevel);
		}
		
		return p_this.center.x;
	}
	
	// Gets the y at a zoom level.  This involves scaling the value by a power of 2.
	function GetY(zoomLevel)
	{
		if (zoomLevel)
		{
			return p_this.center.y * Math.pow(2, zoomLevel - p_this.zoomLevel);
		}
		
		return p_this.center.y;
	}
	
	// Gets the center point
	function GetCenter(zoomLevel)
	{
		if (zoomLevel == undefined)
		{
			return p_this.center;
		}
		
		return new Msn.VE.Pixel(p_this.GetX(zoomLevel), p_this.GetY(zoomLevel));
	}
	
	// Resolves the view into a pixel type view.  Also populates the center latlong object.
	// All of the view types can ultimately be boiled down to a center pixel and zoom level, 
	// given a view mode (which provides latlong to pixel conversion) and the control dimensions.
	function Resolve(viewMode, width, height)
	{
		switch (viewType)
		{
			case "pixel":
				p_this.latlong = viewMode.PixelToLatLong(p_this.center, p_this.zoomLevel);
				break;
			
			case "latlong":
				p_this.center = viewMode.LatLongToPixel(p_this.latlong, p_this.zoomLevel);
				break;
				
			case "pixelRect":
				ResolveRectangle(viewMode, width, height);
				break;
			
			case "latlongRect":
				if (p_this.mapStyle == "o")
				{
					p_this.zoomLevel = 1;
						
					var scene = viewMode.GetScene();
					if (!scene.ContainsLatLong(p_this.latlongRect.northwest) || !scene.ContainsLatLong(p_this.latlongRect.southeast))
					{
						p_this.latlong = p_this.latlongRect.Center();
						p_this.center = viewMode.LatLongToPixel(p_this.latlong, p_this.zoomLevel);
					}
					else
					{
						p_this.pixelRect.topLeft = viewMode.LatLongToPixel(p_this.latlongRect.northwest, p_this.zoomLevel);
						p_this.pixelRect.bottomRight = viewMode.LatLongToPixel(p_this.latlongRect.southeast, p_this.zoomLevel);
						ResolveRectangle(viewMode, width, height);
					}
				}
				else
				{
					p_this.zoomLevel = 12;
					p_this.pixelRect.topLeft = viewMode.LatLongToPixel(p_this.latlongRect.northwest, p_this.zoomLevel);
					p_this.pixelRect.bottomRight = viewMode.LatLongToPixel(p_this.latlongRect.southeast, p_this.zoomLevel);
					ResolveRectangle(viewMode, width, height);
				}
				
				break;			
		}
		
		viewType = "pixel";
	}
	
	// Resolves the pixel rectangle to a center point.  
	function ResolveRectangle(viewMode, width, height)
	{
		var zoomPivot = 19;
		var zoomFactor = Math.pow(2, zoomPivot - p_this.zoomLevel);
		var viewportWidth = MathMax(1, MathAbs(p_this.pixelRect.topLeft.x - p_this.pixelRect.bottomRight.x) * zoomFactor);
		var viewportHeight = MathMax(1, MathAbs(p_this.pixelRect.topLeft.y - p_this.pixelRect.bottomRight.y) * zoomFactor);
		var log2 = Math.log(2);
		
		var horizontalZoom = zoomPivot - Math.ceil(Math.log(viewportWidth / width) / log2);
		var verticalZoom = zoomPivot - Math.ceil(Math.log(viewportHeight / height) / log2);
		var newZoom = MathMin(horizontalZoom, verticalZoom);		
		// Zoom levels can never be less than or equal to zero.
		// If the zoom level drops below 1, set it to the min zoom level which is 1.
		if (newZoom <= 0)
		{
			newZoom = 1;
		}
		
		zoomFactor = Math.pow(2, newZoom - p_this.zoomLevel);
		p_this.center.x = 0.5 * (p_this.pixelRect.topLeft.x + p_this.pixelRect.bottomRight.x) * zoomFactor;
		p_this.center.y = 0.5 * (p_this.pixelRect.topLeft.y + p_this.pixelRect.bottomRight.y) * zoomFactor;
		p_this.zoomLevel = newZoom;
		p_this.latlong = viewMode.PixelToLatLong(p_this.center, p_this.zoomLevel);
	}
	
	this.MakeCopy = MakeCopy;
	this.Copy = Copy;
	this.Equals = Equals;
	this.ToString = ToString;
	this.SetCenter = SetCenter;
	this.SetCenterLatLong = SetCenterLatLong;
	this.SetPixelRectangle = SetPixelRectangle;
	this.SetLatLongRectangle = SetLatLongRectangle;
	this.SetZoomLevel = SetZoomLevel;
	this.SetMapStyle = SetMapStyle;
	this.GetX = GetX;
	this.GetY = GetY;
	this.GetCenter = GetCenter;
	this.Resolve = Resolve;
	this.SetZoomLevel = SetZoomLevel;
};

