//------------------------------------------------------------------------------
// MapControl/Modes/ObliqueMode.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: Codye
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// A subset of the map control functionality that is specific to viewing in ortho 
// mode has been refactored out of the control.  The same functionality has been 
// reimplemented for oblique mode.  (This is conceptually an interface for a view 
// mode that will be implemented for both ortho mode and oblique mode, except 
// JavaScript does not have the notion of an interface.)
//
// Each view mode will include the following functionality:
// 
// * Boundary checking
//   * Determining tile validity
//   * Determining zoom validity
//   * Restricting panning and zooming to the edges of the image
// * Building tile filenames
// * Pixel to latlong & latlong to pixel conversions

// The implementation of view mode for oblique imagery.  Oblique mode also includes
// a few helper methods for determining oblique availality.
function ObliqueMode()
{
	var scene = null;
	var available = false;
	var availableBefore = false;
	var xmlHttp = null;
	var obliqueUrl = null;
	var showOnLoad = false;
	var requestUrl = null;
	var timerId = -1;
	var bounds = [ __mapControlObliqueBounds__ ];
	
	// Initializes the oblique mode
	this.Init = function(url)
	{
		obliqueUrl = url;
	};
	
	// Destroys the oblique mode
	this.Destroy = function()
	{
	};
	
	// Determines if a tile is valid within a scene
	function IsValidTile(tileX, tileY, zoomLevel)
	{
		return scene.IsValidTile(tileX, tileY, zoomLevel);
	}
	
	// Gets the tile filename
	function GetFilename(tileX, tileY, zoom)
	{
		return scene.GetTileFilename(tileX, tileY, zoom);
	}

	// Computes the mercator meters per pixel at a specified zoom level.
	function MetersPerPixel(zoomLevel)
	{
		if (!scene)
		{
			return 0;
		}
		
		var pixel = new Msn.VE.Pixel(MathRound(originX + offsetX + width/2), MathRound(originY + offsetY + height/2));
		var latlong1 = scene.PixelToLatLong(pixel, zoomLevel);
		pixel.x++;
		var latlong2 = scene.PixelToLatLong(pixel, zoomLevel);
		
		var sinLat1 = Math.sin(DegToRad(latlong1.latitude));
		var sinLat2 = Math.sin(DegToRad(latlong2.latitude));
		var metersHeight = earthRadius / 2 * MathAbs(Math.log((1 + sinLat1) / (1 - sinLat1)) - Math.log((1 + sinLat2) / (1 - sinLat2)));
		var metersWidth = earthRadius * MathAbs(DegToRad(latlong1.longitude) - DegToRad(latlong2.longitude));
		
		return Math.sqrt(metersWidth * metersWidth + metersHeight * metersHeight);
	}
	
	// Converts an absolute pixel point to a latitude longitude coordinate.
	function PixelToLatLong(pixel, zoomLevel)
	{
		return scene ? scene.PixelToLatLong(pixel, zoomLevel) : null;
	}
	
	// Converts a latitude longitude coordinate to an absolute pixel point.
	function LatLongToPixel(latlong, zoomLevel)
	{
		return scene ? scene.LatLongToPixel(latlong, zoomLevel) : null;
	}

	// Determines if the user can zoom in with the currently set viewport
	function ValidateZoomLevel(view)
	{
		if (view.zoomLevel < 1)
		{
			view.SetZoomLevel(1);
		}
		else if (view.zoomLevel > 2)
		{
			view.SetZoomLevel(2);
		}
	}
	
	function GetScene()
	{
		return scene;
	}
	
	function GetBounds()
	{
		return scene.GetBounds();
	}
	
	function IsAvailable()
	{
		return available;
	}
	
	function GetEventInfo()
	{
		if (!available)
		{
			return null;
		}
		
		var info = new Object();
		return info;
	}
	
	// Kicks off a server side check for oblique availability.  
	// This request passes up the center latlong
	function UpdateAvailability()
	{
		if (currentView.mapStyle == obliqueStyle)
		{
			return;
		}
		
		//If the zoomLevel is invalid, get out
		if (currentView.zoomLevel <= 0) 
		{
			return;
		}
				
		if (currentView.zoomLevel < MapControl.Features.BirdsEyeAtZoomLevel )
		{		
			scene = null;
			availableBefore = available;
			available = false;
			NotifyListeners();
			return;
		}
		
		RequestSceneAtLatLong(currentView.latlong, "North", false, __obliqueUpdateDelay__);
	}
	
	// Kicks off a server side request for an oblique scene at any location 
	// This request passes up a latlong 
	function RequestSceneAtLatLong(latlong, orientation, show, delay)
	{
		CancelRequest();
		
		showOnLoad = show;
		
		// If the point isn't even in the realm of oblique imagery, 
		// save the time and just notify the listeners.
		if (!InBounds(latlong))
		{
			scene = null;
			availableBefore = available;
			available = false;
			NotifyListeners();
			return;
		}
		
		requestUrl = obliqueUrl + "?ver=3&lat=" + latlong.latitude + "&lon=" + latlong.longitude;
		if (orientation)
		{
			requestUrl += "&o=" + orientation;
		}
		
		if (delay)
		{
			timerId = window.setTimeout(SendRequest, delay);
		}
		else
		{
			SendRequest();
			timerId = -1;
		}
	}
	
	// Kicks off a server side request for one specific scene.
	// This request passes up the image id.
	function RequestScene(id)
	{
		if (scene && scene.GetID() == id)
		{
			return;
		}
	
		CancelRequest();
		showOnLoad = true;
		requestUrl = obliqueUrl + "?ver=3&id=" + id;
		SendRequest();
	}
	
	// Cancels any existing requests.
	function CancelRequest()
	{
		try
		{
			if (timerId != -1)
			{
				window.clearTimeout(timerId);
			}
		}
		catch(ex)
		{
			// swallow, the thread may have completed
		}
		
		timerId = -1;
		
		try
		{
			if (xmlHttp)
			{
				xmlHttp.onreadystatechange = null;
				xmlHttp.abort();
			}
		}
		catch(ex)
		{
			// Ignore any problems
		}
	}
	
	// Sends the request on deck.
	function SendRequest()
	{
		CancelRequest();
		
		if (!requestUrl)
		{
			return;
		}
		
		if(Msn.VE.API != null)
		{
			var network = new VENetwork();
			network.ServiceUrl = requestUrl;
			network.BeginInvoke("EndInvokeImagery", null, APIResponseHandler);	
		}
		else
		{
			if (!xmlHttp)
			{
				xmlHttp = GetXmlHttp();
			}
			
			if (xmlHttp)
			{
				xmlHttp.open("POST", requestUrl, true);
				xmlHttp.onreadystatechange = ResponseHandler;
				xmlHttp.send("");
			}
		}
	}
	
	// Handles the oblique response.  This response handler is shared by UpdateAvailability
	// and RequestScene
	function APIResponseHandler(object)
	{
		if(object != null)
		{
			scene = object;
			available = true;			
		}
		else
		{
			scene = null;
			available = false;					
		}		
		NotifyListeners();		
	}	
	function ResponseHandler()
	{
		// Ignore all states other than the final one
		if (!xmlHttp || xmlHttp.readyState != 4) 
		{
			return;
		}
		
		// Clear the timer
		timerId = -1;
		
		availableBefore = available;
		var code = xmlHttp.responseText;
		if (code)
		{
			try
			{
				scene = eval(code + "EndInvokeImagery();");
				if(scene != null)				
					available = true;
				else
					available = false;				
			}
			catch(ex)
			{
				scene = null;
				available = false;
			}
		}
		else
		{
			scene = null;
			available = false;
		}
		
		NotifyListeners();
	}
	
	// Fires all events.  If we need to show on load, make the appropriate SetView call.
	function NotifyListeners()
	{
		// If we're supposed to show on load, display the oblique.
		if (showOnLoad)
		{
			showOnLoad = false;
			
			if (available)
			{
				var view = preferredView.MakeCopy();
				view.SetMapStyle(obliqueStyle, scene.GetID(), scene.GetOrientation());
				SetView(view);
				
				Fire("onobliquechange");
			}
			else
			{
				// The user requested oblique imagery, but we don't have any for this latlong.  Show the last ortho map style instead
				var view = preferredView.MakeCopy();
				view.SetMapStyle(lastOrthoMapStyle);
				
				// We need a way to determine what the zoom level should be when we pop out into ortho.  Unless we
				// have a lat long rectangle, we'll need to revert back to the last ortho zoom level.
				if (view.GetViewType() != "latlongRect")
				{
					view.SetZoomLevel(lastOrthoZoomLevel);
				}
				
				SetView(view);
				
				Fire("onerror", CreateEvent(currentView.latlong, currentView.zoomLevel, L_ObliqueModeImageNotAvailable_Text ));
			}
		}
		
		// Fire the appropriate events if the availability changed
		if (availableBefore != available)
		{
			if (available)
			{
				Fire("onobliqueenter");
			}
			else
			{
				Fire("onobliqueleave");
			}
		}
	}
	
	// Determines if this latlong is in a set of bounds where we have oblique imagery.
	function InBounds(latlong)
	{
		if (!latlong || !bounds || bounds.length == 0)
		{
			return false;
		}
		
		// Build a quad key at the current latlong
		// Note that we generate the quad key at the zoom level of the
		// first element in the bounds list.
		// (zoom level = length(quad key))
		var quadKey = LatLongToQuadKey(latlong, bounds[0].length);
		
		// Search for the quad key in our list of bounds
		return BinarySearchBounds(quadKey, 0, bounds.length - 1);
	}
	
	// Converts a lat/long to a quad key at the requested zoom level
	function LatLongToQuadKey(latlong, zoomLevel)
	{
		var pixel = orthoMode.LatLongToPixel(latlong, zoomLevel);
		var tileX = MathFloor(pixel.x / tileSize);
		var tileY = MathFloor(pixel.y / tileSize);
	
		var quadKey = "";
		for (var i=zoomLevel; i>0; i--)
		{
			var cell = 0;
			var mask = 1 << (i-1);
			
			if ((tileX & mask) != 0)
			{
				cell++;
			}
			
			if ((tileY & mask) != 0)
			{
				cell += 2;
			}
			
			quadKey += (cell + "");
		}
		
		return quadKey;
	}
	
	// Performs a recursive binary search, looking for the quadKey in the bounds array
	function BinarySearchBounds(quadKey, left, right)
	{
		if (right < left)
			return false;
		
		var middle = MathFloor((left + right) / 2);
		
		if (bounds[middle] == quadKey)
			return true;
		
		if (quadKey < bounds[middle])
			return BinarySearchBounds(quadKey, left, middle-1);
		
		return BinarySearchBounds(quadKey, middle+1, right);
	}

	this.IsValidTile = IsValidTile;
	this.GetFilename = GetFilename;
	this.MetersPerPixel = MetersPerPixel;
	this.PixelToLatLong = PixelToLatLong;
	this.LatLongToPixel = LatLongToPixel;
	this.GetBounds = GetBounds;
	this.ValidateZoomLevel = ValidateZoomLevel;
	this.IsAvailable = IsAvailable;
	this.UpdateAvailability = UpdateAvailability;
	this.CancelRequest = CancelRequest;
	this.GetEventInfo = GetEventInfo;
	this.GetScene = GetScene;
	this.RequestSceneAtLatLong = RequestSceneAtLatLong;
	this.RequestScene = RequestScene;
}