//------------------------------------------------------------------------------
// MapControl/ObliqueScene.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: CodyE
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------

// The ObliqueScene class represents one oblique image.
// The id is a unique identifier for this image that is used when creating
// tile and thumbnail filenames.
// The orientation is the direction that this oblique image is facing in degrees.
// Points is an array of latlong points that define the scene.  It starts in the top left corner
// and goes around clockwise.
// Neighbors is an array of oblique id's for the neighboring images.
// Rotations is an array of oblique id's for different orientations of the same location.
Msn.VE.ObliqueScene = function(id, quadKey, runtimeIndex, orientation, baseZoom, stripe, maxTileX, maxTileY, pixelToLatLongMatrix, latLongToPixelMatrix, neighbors, rotations)
{
	// The size of one tile
	var tileSize = 256;
	
	this.neighborScenes = neighbors;
	
	// The bounds of the image.  It is defined in the top zoom level (1 in this case), so we
	// need to scale down the tile extents.
	var bounds = new Msn.VE.Bounds(1, 2, 0, 0, maxTileX / 2, maxTileY / 2);
	
	// Converts a pixel to a lat/long using the precomputed matrix
	function PixelToLatLong(pixel, zoomLevel)
	{
		var zoomFactor = Math.pow(2, zoomLevel - 2);
		var vector = [[ pixel.x / zoomFactor ], [ pixel.y / zoomFactor ], [ 1 ]];
		var result = MatrixMultiply(pixelToLatLongMatrix, vector);

		var latlong = new Msn.VE.LatLong();
		latlong.longitude = result[0][0] / result[2][0];
		latlong.latitude = result[1][0] / result[2][0];

		return latlong;
	}
	
	// Converts a lat/long to a pixel using the precomputed matrix
	function LatLongToPixel(latlong, zoomLevel)
	{
		var zoomFactor = Math.pow(2, zoomLevel - 2);
		var vector = [[ latlong.longitude ], [ latlong.latitude ], [ 1 ]];
		var result = MatrixMultiply(latLongToPixelMatrix, vector);

		var pixel = new Msn.VE.Pixel();
		pixel.x = result[0][0] / result[2][0] * zoomFactor;
		pixel.y = result[1][0] / result[2][0] * zoomFactor;

		return pixel;
	}
	
	// Determines if the tile is within valid range for a zoom level.
	function IsValidTile(tileX, tileY, zoomLevel)
	{
		if (tileX < 0 || tileY < 0)
		{
			return false;
		}
		
		if (zoomLevel == 1)
		{
			return tileX < (maxTileX / 2) && tileY < (maxTileY / 2);
		}
		
		return tileX < maxTileX && tileY < maxTileY;
	}
	
	// Returns the id of the scene
	function GetID()
	{
		return id;
	}
	
	// Builds a filename for a tile
	function GetTileFilename(tileX, tileY, zoomLevel)
	{
		var tileId = tileY * (zoomLevel == 1 ? maxTileX / 2 : maxTileX) + tileX;
		return "http://c" + (tileX % 2) + ".p" + stripe + ".__obliqueTileServer__o" + quadKey + "-" + runtimeIndex + "-" + (baseZoom + zoomLevel - 2) + "-" + tileId + ".jpeg?g=__obliqueGeneration__";
	}
	
	// Builds a filename for a tile
	function GetThumbnailFilename()
	{
		return "http://thumbs.__obliqueTileServer__ot" + id + ".jpeg?g=__obliqueGeneration__";
	}
	
	// Gets a neighbor
	function GetNeighbor(i)
	{
		var sceneId = neighbors[i];
		if (sceneId == null || sceneId == -1)
		{
			return null;
		}
		
		return new Msn.VE.ObliqueScene(sceneId);
	}
	
	// Gets a rotation
	function GetRotation(i)
	{
		var sceneId = rotations[i];
		if (sceneId == null || sceneId == -1)
		{
			return null;
		}
		
		return new Msn.VE.ObliqueScene(sceneId);
	}
	
	// Gets the orientation of this scene
	function GetOrientation()
	{
		return orientation;
	}
	
	// Gets the tile bounds of this scene
	function GetBounds()
	{
		return bounds;
	}
	
	// Gets the width of the scene
	function GetWidth()
	{
		return maxTileX * tileSize;
	}
	
	// Gets the height of the scene
	function GetHeight()
	{
		return maxTileY * tileSize;
	}
	
	// Determines if this scene contains a latlong
	function ContainsLatLong(latlong)
	{
		if (!latlong)
		{
			return false;
		}
	
		var pixel = LatLongToPixel(latlong, 2);
		return ContainsPixel(pixel, 2);
	}
	
	// Determines if this scene contains a pixel point
	function ContainsPixel(pixel, zoomLevel)
	{
		var zoomFactor = Math.pow(2, 2 - zoomLevel);
		var x = pixel.x * zoomFactor;
		var y = pixel.y * zoomFactor;
		return x >=0 && y >= 0 && x < GetWidth() && y < GetHeight();	
	}

	this.PixelToLatLong = PixelToLatLong;
	this.LatLongToPixel = LatLongToPixel;
	this.IsValidTile = IsValidTile;
	this.GetID = GetID;
	this.GetTileFilename = GetTileFilename;
	this.GetThumbnailFilename = GetThumbnailFilename;
	this.GetNeighbor = GetNeighbor;
	this.GetRotation = GetRotation;
	this.GetOrientation = GetOrientation;
	this.GetBounds = GetBounds;
	this.GetWidth = GetWidth;
	this.GetHeight = GetHeight;
	this.ContainsLatLong = ContainsLatLong;
	this.ContainsPixel = ContainsPixel;
};
