//------------------------------------------------------------------------------
// Validators.js
//
// <copyright from='2004' to='2005' company='Microsoft Corporation'> 
// Copyright (c) Microsoft Corporation. All Rights Reserved. 
// Information Contained Herein is Proprietary and Confidential. 
// </copyright>
//
// Owner: EShinn, keithkin
//
// <header>	$Header$</header>
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Dependencies
//
// None
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Referred to by
//
// ConfigurationItem.js
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Validators
//
// A variety of classes that check and parse input.
//------------------------------------------------------------------------------

// Serialization constants
function VE_SerializationConstants() { } 

// The primary separator - used to specify multiple values.
VE_SerializationConstants.primarySeparator = primarySerializationSeparators.split(",");

// The secondary separator - used to split multiple values within a specified value.
VE_SerializationConstants.secondarySeparator = secondarySerializationSeparators.split(",");

// Validates a particular type
function VE_TypeValidator(type)
{
	this.type = type;
	this.validatorType = "VE_TypeValidator"; // This is necessary - JS loses type info once you assign a method to a subclass prototype constructor.
}

// Returns true if the type of the value & the validator type are the same.
VE_TypeValidator.prototype.ValidValue = function(value)
{
	return VE_TypeValidator._ValidValue(value, this.type);
}

// Returns a default value of null.
VE_TypeValidator.prototype.DefaultValue = function()
{
	return null;
}

// Attempts to translate the given value into an appropriate value for the validator type.
VE_TypeValidator.prototype.ParseValue = function(value)
{
	return VE_TypeValidator._ParseValue(value, this.type);
}

// class method for type validation.
VE_TypeValidator._ValidValue = function(value, type)
{
	// Null is always invalid.
	if (value == null)
	{
		return false;
	}
	
	// if this is an integer...	
	if (type == "int")
	{
		return typeof(value) == "number" && value.toString().indexOf(".") == -1;
	}
	
	return typeof(value) == type;	
}

// class method for value parsing.
VE_TypeValidator._ParseValue = function(value, type)
{
	if (value == null)
	{
		return null;
	}

	var typeOfValue = typeof(value);

	if (typeOfValue == "string")
	{
		value = unescape(value).replace(/^\s*|\s*$/g,""); // unescape & trim any whitespace.
	}
	
	// special case the two specific number types.
	if (type == "int" && typeOfValue == "number")
	{
		// we just don't accept FP if this is an int.
		if (value.toString().indexOf(".") > -1)
		{
			return null;
		}

		return value;
	}
	
	if (typeOfValue == type)
	{
		return value;
	}
	
	if (typeOfValue == "string")
	{
		if (type == "int")
		{
			// If there's a period, this is not an int.
			if (value.toString().indexOf(".") > -1)
			{
				return null;
			}
			
			var returnedNumber = parseInt(value);
			if (isNaN(returnedNumber))
			{
				return null;
			}
			
			return returnedNumber;
		}
		else if (type == "number")
		{
			var returnedNumber = parseFloat(value);
			if (isNaN(returnedNumber)) 
			{
				returnedNumber = parseInt(value);
			}
			if (isNaN(returnedNumber)) 
			{
				return null;
			}
			return returnedNumber;
		}
		else if (type == "boolean")
		{
			if (value.toLowerCase() == "true") 
			{
				return true;
			}
			else if (value.toLowerCase() == "false") 
			{
				return false;
			}
		}
	}
	return null;
}

// VE_StringValidator derives from VE_TypeValidator
VE_StringLengthValidator.prototype = new VE_TypeValidator();

// Ensures a string stays within a given length.
// If no bound is defined, string can be infinitely long.
function VE_StringLengthValidator(maxLength)
{
	this.type = "string";
	this.validatorType = "VE_StringValidator";
			
	if (maxLength && maxLength.constructor == Number & maxLength >= 0)
	{
		this.maxLength = maxLength;
	}
}

// Parses a passed in value.  Truncates to max length if it is too long.
VE_StringLengthValidator.prototype.ParseValue = function(value)
{
	var parsedValue = VE_TypeValidator._ParseValue(value, "string");
	// If we have a defined upper bound on length and the parsed string is longer than that, 
	// truncate it to the max length.
	if (parsedValue != null && this.maxLength != null && parsedValue.length > this.maxLength)
	{
		parsedValue = parsedValue.substr(0, this.maxLength);
	}
	
	return parsedValue;
}

// Does type checking as well as return false if there's a defined upper bound length
// and the passed in value exceeds it.
VE_StringLengthValidator.prototype.ValidValue = function(value)
{
	if (!VE_TypeValidator._ValidValue(value, "string"))
	{
		return false;
	}

	if (this.maxLength != null && value.length > this.maxLength)
	{
		return false;
	}

	return true;
}

// Returns an empty string.
VE_StringLengthValidator.prototype.DefaultValue = function(value)
{
	return "";
}

// VE_ArrayValidator derives from VE_TypeValidator
VE_ArrayValidator.prototype = new VE_TypeValidator();

// Validates array values.  Can set max & min lengths on the array
// as well as define an optional array value validator.
// An undefined max length represents infinity.
function VE_ArrayValidator(type, minValues, maxValues, allowNull, valueValidationFunction, valueParsingFunction)
{
	this.type = type;
	this.validatorType = "VE_ArrayValidator";
	this.allowNull = allowNull;
	
	if (valueValidationFunction != null && typeof(valueValidationFunction) == "function")
	{
		this.valueValidationFunction = valueValidationFunction;
	}
	if (valueParsingFunction != null && typeof(valueParsingFunction) == "function")
	{
		this.valueParsingFunction = valueParsingFunction;
	}
	
	if (minValues && minValues.constructor == Number & minValues >= 0)
	{
		this.minValues = minValues;
	}
	else
	{
		this.minValues = 0;
	}
		
	if (maxValues && maxValues.constructor == Number & maxValues >= 0)
	{
		this.maxValues = maxValues;
	}
	
	// swap if both defined & min is greater than max
	if (this.maxValues != null && this.minValues > this.maxValues)
	{
		var oldMinValue = this.minValues;
		this.minValues = this.maxValues;
		this.maxValues = oldMinValue;
	}
}

// Returns the valid values in an array or attempts to construct an array from a string.
VE_ArrayValidator.prototype.ParseValue = function(value, apiVersion)
{
	if (value == null)
	{
		return null;
	}

	// Set it to the current URL API Version if it hasn't been decided yet...
	// Normally, we would make a call to state at this point, but we don't know when this is getting
	// called so state might not exist at this point in time.
	if (apiVersion == null)
	{
		apiVersion = currentUrlApiVersion;
	}

	// Get the specified versions of the separators	
	primarySeparator = VE_SerializationConstants.primarySeparator[apiVersion];
	
	var arrayValues = null;
	
	if (value.constructor == Array)
	{
		arrayValues = new Array();
		for (var i = 0; i < value.length; i++)
		{
			var parsedValue = this._ParseSingleValue(value[i], apiVersion);
			if ((this.allowNull && parsedValue == null) || VE_TypeValidator._ValidValue(parsedValue, this.type))
			{
				arrayValues.push(parsedValue);
			}
		}
	}
	else if (value.constructor == String)
	{
		arrayValues = new Array();
		// Some sites may escape the primary separator.  Fix that.
		value = value.replace(eval("/" + IOSec.EncodeUrl(primarySeparator) + "/gi"), primarySeparator);
		var valueStrings = value.split(primarySeparator);
		for (var i = 0; i < valueStrings.length; i++)
		{
			// Ignore empty strings.
			if (!this.allowNull && !valueStrings[i])
			{
				continue;
			}
			
			var parsedValue = this._ParseSingleValue(valueStrings[i], apiVersion);
			if ((this.allowNull && parsedValue == null) || VE_TypeValidator._ValidValue(parsedValue, this.type))
			{
				arrayValues.push(parsedValue);
			}
		}
	}
	
	// Truncate array if it is too long.
	if (arrayValues != null && this.maxValues != null && arrayValues.length > this.maxValues)
	{
		arrayValues.splice(this.maxValues, arrayValues.length - this.maxValues);
	}
	
	return arrayValues;
}

// Validates that the passed in value is an array, 
// the number of elements in this array fall within the length bounds defined
// and all the elements are of the correct type
VE_ArrayValidator.prototype.ValidValue = function(value, apiVersion)
{
	// Set it to the current URL API Version if it hasn't been decided yet...
	// Normally, we would make a call to state at this point, but we don't know when this is getting
	// called so state might not exist at this point in time.
	if (apiVersion == null)
	{
		apiVersion = currentUrlApiVersion;
	}

	if (!value || value.constructor != Array)
	{
		return false;
	}

	if (value.length < this.minValues || (this.maxValues != null && value.length > this.maxValues))
	{
		return false;
	}
	
	for (var i = 0; i < value.length; i++)
	{
		if ((value[i] == null && !this.allowNull) || (value[i] != null && typeof(value[i]) != this.type))
		{
			return false;
		}
	}
	
	if (this.valueValidationFunction)
	{
		if (!this.valueValidationFunction(value, apiVersion))
		{
			return false;
		}
	}
	
	return true;
}

// Returns an empty array.
VE_ArrayValidator.prototype.DefaultValue = function()
{
	return [];
}

// Parses a single value.
VE_ArrayValidator.prototype._ParseSingleValue = function(value, apiVersion)
{
	// Use provided parsing function if present.
	if (this.valueParsingFunction != null)
	{
		return this.valueParsingFunction(value, apiVersion);
	}
	// Otherwise, use default.
	else
	{
		return VE_TypeValidator._ParseValue(value, this.type);
	}
}


// VE_ValidValuesValidator extends VE_TypeValidator
VE_ValidValuesValidator.prototype = new VE_TypeValidator();

// Validates a value within a provided set of values - does not support arrays.
function VE_ValidValuesValidator(type, validValues)
{
	this.type = type;
	this.validatorType = "VE_ValidValuesValidator"; 	// This is necessary - JS loses type info once you assign a method to a subclass prototype constructor.
	
	this.validValues = new Array();
	if (this.type == "boolean")
	{
		this.validValues.push(true);
		this.validValues.push(false);
	}
	else if (validValues && validValues.constructor == Array)
	{
		for (var i = 0; i < validValues.length; i++)
		{
			if (VE_TypeValidator._ValidValue(validValues[i], this.type))
			{
				this.validValues.push(validValues[i]);
			}
		}
	}
}

// Returns true if the types match & the supplied value is in the list of valid values
VE_ValidValuesValidator.prototype.ValidValue = function(value)
{
	if (!VE_TypeValidator._ValidValue(value, this.type))
	{
		return false;
	}
	// No valid values is interpreted as any type valid values goes.
	if (this.validValues.length == 0)
	{
		return true;
	}
	
	for (var i = 0; i < this.validValues.length; i++)
	{
		if (value == this.validValues[i]) 
		{
			return true;
		}
	}
	return false;
}

// Returns the first valid value if it exists.  Otherwise, return null;
VE_ValidValuesValidator.prototype.DefaultValue = function()
{
	if (this.validValues.length > 0)
	{
		return this.validValues[0];
	}
	else 
	{
		return null;
	}
}

// VE_RangeValidator extends VE_TypeValidator
VE_RangeValidator.prototype = new VE_TypeValidator();

// Validates that a value falls within a particular range - Only supports numeric types.
function VE_RangeValidator(minValue, maxValue, integerOnly)
{
	if (integerOnly)
	{
		this.type = "int";
	}
	else
	{
		this.type = "number";
	}
	this.multipleValues = false;
	this.validatorType = "VE_RangeValidator"; 	// This is necessary - JS loses type info once you assign a method to a subclass prototype constructor.
	
	if (VE_TypeValidator._ValidValue(minValue, this.type))
	{
		this.minValue = minValue;
	}
	
	if (VE_TypeValidator._ValidValue(maxValue, this.type))
	{
		this.maxValue = maxValue;
	}

	// If we have both a min and a max and the min is greater than the max, swap them.
	if (this.minValue && this.maxValue && this.minValue > this.maxValue)
	{
		var oldMinValue = this.minValue;
		this.minValue = this.maxValue;
		this.maxValue = oldMinValue;
	}
}

// Returns true if types match & value falls within specified range.
// (Null boundary implies inifinite)
VE_RangeValidator.prototype.ValidValue = function(value)
{
	if (!VE_TypeValidator._ValidValue(value, "number")) 
	{
		return false;
	}
	
	if (this.minValue != null && value < this.minValue) 
	{
		return false;
	}
	if (this.maxValue != null && value > this.maxValue)
	{
		return false;
	}
	
	return true;
}

// Returns the minimum value if defined, then max if defined, then null.
VE_RangeValidator.prototype.DefaultValue = function()
{
	if (this.minValue != null)
	{
		return this.minValue;
	}
	else if (this.maxValue != null)
	{
		return this.maxValue;
	}
	else
	{
		return null;
	}
}

// VELatLongValidator
// Validates Latitude/Longitude state
// This validator can handle traditional Latitude/Longitude state as well
// as encoded Latitude/Longitude state.
function VELatLongValidator()
{
	var m_arrayValidator = new VE_ArrayValidator("number", 2, 2, false);
	var m_latValidator = new VE_RangeValidator(-85.0, 85.0);
	var m_lonValidator = new VE_RangeValidator(-180.0, 180.0);
	
	//ValidValue
	//returns true if the passed in value is valid. Returns false otherwise.
	this.ValidValue = function(value, apiVersion)
	{
		if (value == null)
		{
			return false;
		}

		var isValid = false;

		// Set it to the current URL API Version if it hasn't been decided yet
		// Normally, we would make a call to state at this point, but we don't know when this is getting
		// called so state might not exist at this point in time.
		if (apiVersion == null)
		{
			apiVersion = currentUrlApiVersion;
		}

		var latLon = value;
		
		// Get the specific version of the separator
		var primarySeparator = VE_SerializationConstants.primarySeparator[apiVersion];

		//If the value is an Array or a String with a primary separator in it,
		//validate it with the array validator
		if ( (value.constructor == Array) || 
			 ((value.constructor == String) && (value.match(primarySeparator) != null)) )
		{
			isValid = m_arrayValidator.ValidValue(value)
		}
		//otherwise if it is just a String, then it is an encoded lat/long value.
		//Decode it and validate it.
		else if (value.constructor == String)
		{
			try
			{
				var encoder = new VELatLongEncoding();
				latLon = encoder.Decode(value);
				//If the decoder did not throw an exception, then it is a valid encoding
				isValid = true;
			}
			catch(ex)
			{
				//If the decoder threw an exception, then this is an invalid encoding
				isValid = false;
			}
		}

		//If everything is ok so far, then validate the range of the lat/long coordinates
		if (isValid)
		{
			isValid = m_latValidator.ValidValue(latLon[0]) && m_lonValidator.ValidValue(latLon[1]);
		}

		return isValid;
	}
	
	//ParseValue
	//returns a parsed value of the passed in lat/long value
	this.ParseValue = function(value, apiVersion)
	{
		// Set it to the current URL API Version if it hasn't been decided yet...
		// Normally, we would make a call to state at this point, but we don't know when this is getting
		// called so state might not exist at this point in time.
		if (apiVersion == null)
		{
			apiVersion = currentUrlApiVersion;
		}

		// Get the specific version of the separator
		var primarySeparator = VE_SerializationConstants.primarySeparator[apiVersion];

		//If the value is an Array or a String with a primary separator in it,
		//then parse the value with the array validator since it is not encoded.
		if ( (value.constructor == Array) || 
			 ((value.constructor == String) && (value.match(primarySeparator) != null)) )
		{
			return m_arrayValidator.ParseValue(value, apiVersion);
		}
		else 
		{
			//this is an encoded lat/long value that does not need to be parsed.
			//Go ahead and return the value.
			return value;
		}
	}

	//DefaultValue
	this.DefaultValue = function()
	{
		//return the default latitude, longitude for user market
		return [eval(defaultLatitude),eval(defaultLongitude)];
	}
}
